import { defineEventHandler } from 'h3';
import { mount } from '@vue/test-utils';
import { h as h$1, Suspense, nextTick } from 'vue';
import { defu } from 'defu';
import { defineComponent, useRouter, h } from '#imports';
import NuxtRoot from '#build/root-component.mjs';

function registerEndpoint(url, handler) {
  if (!window.__app)
    return;
  window.__app.use("/_" + url, defineEventHandler(handler));
  window.__registry.add(url);
}
function mockNuxtImport(_name, _factory) {
  throw new Error(
    "mockNuxtImport() is a macro and it did not get transpiled, this may be an internal bug of nuxt-vitest."
  );
}
function mockComponent(_path, _component) {
  throw new Error(
    "mockComponent() is a macro and it did not get transpiled, this may be an internal bug of nuxt-vitest."
  );
}

const RouterLink = defineComponent({
  functional: true,
  props: {
    to: [String, Object],
    custom: Boolean,
    replace: Boolean,
    // Not implemented
    activeClass: String,
    exactActiveClass: String,
    ariaCurrentValue: String
  },
  setup: (props, { slots }) => {
    const navigate = () => {
    };
    return () => {
      const route = props.to ? useRouter().resolve(props.to) : {};
      return props.custom ? slots.default?.({ href: props.to, navigate, route }) : h(
        "a",
        {
          href: props.to,
          onClick: (e) => {
            e.preventDefault();
            return navigate();
          }
        },
        slots
      );
    };
  }
});

async function mountSuspended(component, options) {
  const {
    props = {},
    attrs = {},
    slots = {},
    route = "/",
    ..._options
  } = options || {};
  const vueApp = globalThis.__unctx__.get("nuxt-app").tryUse().vueApp;
  const { render, setup } = component;
  let setupContext;
  return new Promise((resolve) => {
    const vm = mount(
      {
        setup: (props2, ctx) => {
          setupContext = ctx;
          return NuxtRoot.setup(props2, {
            ...ctx,
            expose: () => {
            }
          });
        },
        render: (renderContext) => h$1(
          Suspense,
          { onResolve: () => nextTick().then(() => resolve(vm)) },
          {
            default: () => h$1({
              async setup() {
                const router = useRouter();
                await router.replace(route);
                const clonedComponent = {
                  ...component,
                  render: render ? (_ctx, ...args) => render(renderContext, ...args) : void 0,
                  setup: setup ? (props2) => setup(props2, setupContext) : void 0
                };
                return () => h$1(clonedComponent, { ...props, ...attrs }, slots);
              }
            })
          }
        )
      },
      defu(_options, {
        slots,
        global: {
          config: {
            globalProperties: vueApp.config.globalProperties
          },
          provide: vueApp._context.provides,
          components: { RouterLink }
        }
      })
    );
  });
}

const WRAPPER_EL_ID = "test-wrapper";
async function renderSuspended(component, options) {
  const {
    props = {},
    attrs = {},
    slots = {},
    route = "/",
    ..._options
  } = options || {};
  const { render: renderFromTestingLibrary } = await import('@testing-library/vue');
  const { vueApp } = globalThis.__unctx__.get("nuxt-app").tryUse();
  const { render, setup } = component;
  let setupContext;
  return new Promise((resolve) => {
    const utils = renderFromTestingLibrary(
      {
        // eslint-disable-next-line @typescript-eslint/no-shadow
        setup: (props2, ctx) => {
          setupContext = ctx;
          return NuxtRoot.setup(props2, {
            ...ctx,
            expose: () => {
            }
          });
        },
        render: (renderContext) => (
          // See discussions in https://github.com/testing-library/vue-testing-library/issues/230
          // we add this additional root element because otherwise testing-library breaks
          // because there's no root element while Suspense is resolving
          h$1(
            "div",
            { id: WRAPPER_EL_ID },
            h$1(
              Suspense,
              { onResolve: () => nextTick().then(() => resolve(utils)) },
              {
                default: () => h$1({
                  async setup() {
                    const router = useRouter();
                    await router.replace(route);
                    const clonedComponent = {
                      ...component,
                      render: render ? (_ctx, ...args) => render(renderContext, ...args) : void 0,
                      setup: setup ? (
                        // eslint-disable-next-line @typescript-eslint/no-shadow
                        (props2) => setup(props2, setupContext)
                      ) : void 0
                    };
                    return () => h$1(clonedComponent, { ...props, ...attrs }, slots);
                  }
                })
              }
            )
          )
        )
      },
      defu(_options, {
        slots,
        global: {
          config: {
            globalProperties: vueApp.config.globalProperties
          },
          provide: vueApp._context.provides,
          components: { RouterLink }
        }
      })
    );
  });
}

export { mockComponent, mockNuxtImport, mountSuspended, registerEndpoint, renderSuspended };
